/* mpTree 0.1.2
 * -----------[ http://mptree.netpedia.net
 *
 *   Use this software at your own risk!
 *    This software falls under the GPL.
 *   Read COPYING for further informations.
 */

#include "mptree.h"
#include <math.h>

void create__tree(GtkWidget * treetab)
{
    char *newdb[]= {"New Database", };

    ctree = gtk_ctree_new(1, 0);
    gtk_clist_set_row_height (GTK_CLIST(ctree), 16);
    gtk_clist_set_column_auto_resize (GTK_CLIST(ctree), 0, TRUE);

    gtk_signal_connect(GTK_OBJECT(ctree), "tree_select_row",
		       GTK_SIGNAL_FUNC(tree_item_selected), ctree);
    gtk_signal_connect(GTK_OBJECT(ctree), "button_press_event",
    		       GTK_SIGNAL_FUNC(tree_rightmouse_pressed), NULL);  

    root = gtk_ctree_insert_node(GTK_CTREE(ctree), NULL, NULL, newdb, 5,
				 GTK_PIXMAP(dbicon)->pixmap, NULL,
				 GTK_PIXMAP(dbicon)->pixmap, NULL,
				 FALSE, TRUE);

    gtk_container_add(GTK_CONTAINER(treetab), ctree);
}

unsigned int add_files_recursive(char *d, char *relroot, char media)
{
    DIR *dp;
    additional_info *ainfo;
    ID3_Data *cur_tag;
    struct dirent *dir;
    struct stat ent_sbuf;
    gchar *name;
    gchar *path, *relpath;
    unsigned int songcount = 0;
    char *text[2];
    char trackn;

    text[1] = NULL;

    if (!strcmp(d, relroot)) gtk_widget_hide(ctree);
    if ((dp = opendir(d))==NULL) {
        /* dir not found */
        return 0;
    }
    while ((dir = readdir(dp)) != NULL) {
        /* skips removed files */
        if (dir->d_ino > 0) {
            name = dir->d_name;
            path = g_strconcat(d, "/", name, NULL);

            if ((strcmp(name, ".")) && (strcmp(name, "..")))    // don't scan ./.. dirs

            {
                if (stat(path, &ent_sbuf) >= 0 && S_ISDIR(ent_sbuf.st_mode)) {
                    songcount = songcount + add_files_recursive(path, relroot, media);
                } else {
                    if ((!strcmp(name + strlen(name) - 4, ".mp3")) ||
                        !strcmp(name + strlen(name) - 4, ".MP3")) {

                        cur_tag = get_id3_tag(path);
                        if (cur_tag) {
			    media_size = media_size + (float) ent_sbuf.st_size / 1048576; // 1024x1024
                            relpath = path + strlen(relroot) + 1;       // relative path stored without "./"

                            if (!cur_tag->artist)
                                cur_tag->artist = NO_ARTIST;
                            if (!cur_tag->album)
                                cur_tag->album = NO_ALBUM;

                            // parsing single song

                            songcount = songcount + 1;

                            if (!cur_tag->title)
                                cur_tag->title = NO_TITLE;
                            ainfo = (additional_info *) malloc(sizeof(additional_info));
                            ainfo->s_length = song_length(path);        // length in seconds
			    media_sl = media_sl + ainfo->s_length;

                            ainfo->filename = (char *) malloc(strlen(path) + 1);
                            strcpy(ainfo->filename, relpath);
			    trackn = get_tracknumber(relpath);

		   	    if (trackn) {
			      text[0] = (char *) malloc (strlen(cur_tag->title) + 5);
			      sprintf(text[0], "%02d. %s", trackn, cur_tag->title);
			      ainfo->has_trackn = TRUE;
			    }
			    else {
			      text[0] = cur_tag->title;
			      ainfo->has_trackn = FALSE;
			    }
                            ainfo->year = NULL;
			    ainfo->artist = NULL;
                            if ((cur_tag->year) && (strlen(cur_tag->year) == 4)) {
                                ainfo->year = (char *) malloc(strlen(cur_tag->year) + 1);
                                strcpy(ainfo->year, cur_tag->year);
                            }
                            ainfo->comment = NULL;
                            if ((cur_tag->comment != NULL) && (strlen(cur_tag->comment) <=30)) {
                                ainfo->comment = (char *) malloc(strlen(cur_tag->comment) + 1);
                                strcpy(ainfo->comment, cur_tag->comment);
                            }
                            ainfo->onmedia = media;
                            ainfo->genre = cur_tag->genre;
			    
			    add_song_to_tree((GtkCTree *) ctree, root, cur_tag->artist, cur_tag->album,
			    		text[0], ainfo);
			    
                            free(cur_tag);
                        }
                    }
                }
            }
            g_free(path);
        }
    }
    closedir(dp);
    if (!strcmp(d, relroot)) gtk_widget_show(ctree);
    return songcount;
}

void sort_soundtracks(unsigned short media)
{
    dir_list *first = NULL, *cur_dirlist;
    artist_list *cur_alist;
    GtkCTreeNode *c_artist, *c_album, *c_song, *sndtrack, *tmp_song=NULL, *tmp_album=NULL, *tmp_artist=NULL;
    additional_info *ainfo;
    char *just_path, *cur_artist, *new_text;
    float calc;
    char *text[2];

    c_artist = GTK_CTREE_ROW(root)->children;
    text[1] = NULL;
    gtk_widget_hide(ctree);	
    do {
     if (c_artist != sndt_root)
     {
        c_album = GTK_CTREE_ROW(c_artist)->children;
        do {
            c_song = GTK_CTREE_ROW(c_album)->children;
            do {
                ainfo = gtk_ctree_node_get_row_data(GTK_CTREE(ctree), c_song);
                if (media == ainfo->onmedia) {
                    just_path = get_jpath(ainfo->filename);
                    cur_dirlist = get_dirlist_elem(first, just_path);
                    if (!cur_dirlist) {
                        cur_dirlist = (dir_list *) malloc(sizeof(dir_list));
                        cur_dirlist->dirname = just_path;
                        cur_dirlist->f_artist = NULL;
                        cur_dirlist->songcount = 0;
                        cur_dirlist->next = first;
                        first = cur_dirlist;
                    }
                    cur_artist = get_artist_by_node(c_song);
                    if (cur_dirlist->f_artist != NULL) {
                        cur_alist = get_artistlist_elem(cur_dirlist->f_artist, cur_artist);
                    } else {
                        cur_alist = NULL;
                    }
                    if (!cur_alist) {
                        cur_alist = (artist_list *) malloc(sizeof(artist_list));
                        cur_alist->artist = cur_artist;
                        if (cur_dirlist->f_artist == NULL)
                            cur_dirlist->mostSongs = cur_alist;
                        cur_alist->next = cur_dirlist->f_artist;
                        cur_alist->songcount = 0;
                        cur_dirlist->f_artist = cur_alist;
                    }
                    cur_alist->songcount = cur_alist->songcount + 1;
                    cur_dirlist->songcount = cur_dirlist->songcount + 1;
                    if (cur_alist->songcount > cur_dirlist->mostSongs->songcount)
                        cur_dirlist->mostSongs = cur_alist;     // current artist has most songs in list
                }
                c_song = GTK_CTREE_ROW(c_song)->sibling;
            } while (c_song);
            c_album = GTK_CTREE_ROW(c_album)->sibling;
        } while (c_album);
      }
      c_artist = GTK_CTREE_ROW(c_artist)->sibling;
    } while (c_artist);

    cur_dirlist = first;
    while (cur_dirlist) {
        cur_alist = cur_dirlist->f_artist;
        if (cur_dirlist->mostSongs->songcount != cur_dirlist->songcount) {
            calc = (100 / cur_dirlist->songcount) * cur_dirlist->mostSongs->songcount;
            if (calc < MAX_PERCENT) {
//              printf("%s - most songs by: %s, %.2f%%\n", cur_dirlist->dirname, cur_dirlist->mostSongs->artist, calc);
    		if (!sndt_root) {
		    char *text[2];

   		    text[1] = NULL;
   		    text[0] = SNDTRACKS;
   		    sndt_root = gtk_ctree_insert_node(GTK_CTREE(ctree), root, NULL, text, 5,
                                    GTK_PIXMAP(artisticon)->pixmap, NULL,
                                    GTK_PIXMAP(artisticon)->pixmap, NULL,
                                    FALSE, FALSE);

    		}
                text[0] = cur_dirlist->dirname;
                sndtrack = gtk_ctree_insert_node(GTK_CTREE(ctree), sndt_root, NULL, text, 5,
                                     GTK_PIXMAP(albumicon)->pixmap, NULL,
                                     GTK_PIXMAP(albumicon)->pixmap, NULL,
                                     FALSE, FALSE);
                c_artist = GTK_CTREE_ROW(root)->children;
                do {
                  if (c_artist != sndt_root)
                  {
                    c_album = GTK_CTREE_ROW(c_artist)->children;
                    do {
			tmp_album = GTK_CTREE_ROW(c_album)->sibling;
                        c_song = GTK_CTREE_ROW(c_album)->children;
                        do {
                          ainfo = gtk_ctree_node_get_row_data(GTK_CTREE(ctree), c_song);
                          tmp_song=GTK_CTREE_ROW(c_song)->sibling;
                          if (ainfo->onmedia == media)
                          {
                            if ((!strncmp(ainfo->filename, cur_dirlist->dirname, strlen(cur_dirlist->dirname))) 
//					&& (*(ainfo->filename+strlen(cur_dirlist->dirname)) == '/'))
		// BETTER:
					&& (strlen(cur_dirlist->dirname) == strlen(get_jpath(ainfo->filename))))
                            {
			        if (ainfo->has_trackn) {
 				  new_text = (char *) malloc (strlen(get_title(c_song))
				  	+strlen(get_artist_by_node(c_song))+8);
				  sprintf(new_text, "%02d. %s [%s]", get_tracknumber(ainfo->filename), 
				  	get_title(c_song), get_artist_by_node(c_song));
				  ainfo->artist = new_text+strlen(get_title(c_song))+6;
				} else {
				  new_text = (char *) malloc (strlen(get_title(c_song))
				  	+strlen(get_artist_by_node(c_song))+4);
				  sprintf(new_text, "%s [%s]", get_title(c_song), 
				  	get_artist_by_node(c_song));
				  ainfo->artist = new_text+strlen(get_title(c_song))+2;
				}
				text[0] = new_text;
				gtk_ctree_remove_node(GTK_CTREE(ctree), c_song);
				c_song = gtk_ctree_insert_node(GTK_CTREE(ctree), sndtrack, NULL, text, 5,
                                     GTK_PIXMAP(songicon)->pixmap, NULL,
                                     GTK_PIXMAP(songicon)->pixmap, NULL,
                                                           FALSE, FALSE);
                            	gtk_ctree_node_set_row_data(GTK_CTREE(ctree), c_song, ainfo);
                            }
                          }
                          c_song = tmp_song;
                        } while (c_song);
                        if ((GTK_CTREE_ROW(c_album)->children)==NULL)
                                gtk_ctree_remove_node(GTK_CTREE(ctree),c_album);
                        c_album = tmp_album;
                    } while (c_album);
                  }
                  tmp_artist=GTK_CTREE_ROW(c_artist)->sibling;
                  if ((GTK_CTREE_ROW(c_artist)->children)==NULL) {
                        gtk_ctree_remove_node(GTK_CTREE(ctree),c_artist);
                  }
                  c_artist = tmp_artist;
                } while (c_artist);
            }
        }
        cur_dirlist = cur_dirlist->next;
  }
//  free_dirlist(first);
  gtk_widget_show(ctree);
}

void free_dirlist(dir_list *first)
{
  dir_list *dl, *nd;
  artist_list *al, *na;

  dl = first;
  while(dl)
  {
    al = dl->f_artist;
    do {
      na = al->next;
      free(al);
      al = na;
    } while(al);
    nd = dl->next;
    free(dl->dirname);
    free(dl);
    dl = nd;
  }
  free(first);
}

GtkCTreeNode *my_find_tree_node(GtkCTreeNode * node, char *text)
 {
    GtkCTreeNode *temp;

    temp = GTK_CTREE_ROW(node)->children;
    if (!temp)
        return NULL;
    while (temp) {
        if (!strcasecmp(text, GTK_CELL_TEXT(GTK_CTREE_ROW(temp)->row.cell[0])->text))
            return temp;
        temp = GTK_CTREE_ROW(temp)->sibling;
    }
    return NULL;
}

char *get_node_text(GtkCTreeNode *node)
{
  return GTK_CELL_TEXT(GTK_CTREE_ROW(node)->row.cell[0])->text;
}

char *get_title(GtkCTreeNode *song)
{
  char *title, *a, *sndt_sn;
  GtkCTreeNode *parent;
  additional_info *ainfo;

  parent = GTK_CTREE_ROW(song)->parent;
  parent = GTK_CTREE_ROW(parent)->parent;
  title = GTK_CELL_TEXT(GTK_CTREE_ROW(song)->row.cell[0])->text;
  ainfo = gtk_ctree_node_get_row_data(GTK_CTREE(ctree), song);

  if (ainfo->has_trackn) {
    title = title+4;
  }
  if (ainfo->artist)  
  {
    a = get_artist_by_node(song);
    sndt_sn = (char *) malloc (strlen(title)-strlen(a)-2);
    strncpy(sndt_sn, title, strlen(title)-strlen(a)-2);
    *(sndt_sn+strlen(title)-strlen(a)-3) = '\0';
    return sndt_sn; 
  }
  else {
    return title;
  }
}

char *get_artist_by_node(GtkCTreeNode * song)
{
    GtkCTreeNode *parent;
    char *artist, *node_ap;
    additional_info *ainfo;

    ainfo = gtk_ctree_node_get_row_data(GTK_CTREE(ctree), song);
    parent = GTK_CTREE_ROW(song)->parent;
    parent = GTK_CTREE_ROW(parent)->parent;
    if (ainfo->artist)
    {
       node_ap = ainfo->artist;
       artist = (char *) malloc (strlen(node_ap));
       strncpy(artist, node_ap, strlen(node_ap)-1);
       *(artist+strlen(node_ap)-1) = '\0';

       return artist;
    } else
    	return GTK_CELL_TEXT(GTK_CTREE_ROW(parent)->row.cell[0])->text;	// returns pointer to artist
}

dir_list *get_dirlist_elem(dir_list * first, char *just_path)
{
    dir_list *cur;

    if (!first)
        return NULL;            // list empty

    cur = first;
    while (cur) {
        if (!strcmp(cur->dirname, just_path)) {
            return cur;
        }
        cur = cur->next;
    }
    return NULL;
}

artist_list *get_artistlist_elem(artist_list * first, char *artist)
{
    artist_list *cur;

    cur = first;
    while (cur) {
        if (cur->artist == artist) {
            return cur;
        }
        cur = cur->next;
    }
    return NULL;
}

char *get_jpath(char *filename)
{
    char *p;
    static char *jp;

    p = filename + strlen(filename);
    while (*(p) != '/') {
        p = p - 1;
    }
    jp = (char *) malloc(strlen(filename) - strlen(p)+1);
    strncpy(jp, filename, strlen(filename) - strlen(p));
    *(jp+strlen(filename) - strlen(p))='\0';
    return jp;
}

void tree_item_selected(GtkWidget * tree, GtkWidget * node)
{
    additional_info *info;
    char days, hours, minutes, seconds;

    if ((GtkCTreeNode *) node==root || (GtkCTreeNode *) node==av_root) return; 
    else if ((info = gtk_ctree_node_get_row_data(GTK_CTREE(tree), GTK_CTREE_NODE(node)))) {
	dhms_time(info->s_length, &days, &hours, &minutes, &seconds);
	sprintf(message, "[%s] (%s|%s), length: %02d:%02d", info->year, get_media_name(info->onmedia), info->filename, (hours+days*24)*60+minutes, seconds);
	gtk_statusbar_push(GTK_STATUSBAR(statusbar), 1, message);
    }
    else if(GTK_CTREE_ROW( GTK_CTREE_ROW(node)->children )->children) { 
    }
    else { 
    }
    selected_tree_item = GTK_CTREE_NODE(node);
}

void tree_rightmouse_pressed(GtkWidget *widget, GdkEventButton *event)
{
    if (event->button == 3) //right Mousebutton
    {   
        GtkItemFactory *popup_itemfactory;
	GtkWidget *popmenu;
	GtkAccelGroup *accel_group;
        GtkCTreeNode *node;
	gint row=-1, col, numb_of_items;
	GtkItemFactoryEntry menu_items[] = {
	   { "/Add Item to _songList", NULL, to_songlist, 0, NULL },
	   { "/Add Item to _GQmpeg's Playlist", "<control>A", add_to_external_playlist, 0, NULL }
	};

	gtk_clist_get_selection_info (GTK_CLIST(widget), event->x, event->y, &row, &col);
	node = gtk_ctree_node_nth(GTK_CTREE(widget), (guint)row);
	if (row == -1 || !node) return;

	gtk_ctree_select(GTK_CTREE(widget), node);
	selected_tree_item=GTK_CTREE_NODE(node);

	numb_of_items = sizeof (menu_items) / sizeof(menu_items[0]);
	accel_group = gtk_accel_group_new ();
	popup_itemfactory = gtk_item_factory_new (GTK_TYPE_MENU, "<treepopup>", accel_group);
	gtk_item_factory_create_items (popup_itemfactory, numb_of_items, menu_items, NULL);
	gtk_accel_group_attach (accel_group, GTK_OBJECT (mainwin));
        popmenu = gtk_item_factory_get_widget (popup_itemfactory, "<treepopup>");
	
	gtk_menu_popup( GTK_MENU(popmenu), NULL, NULL, NULL, NULL, event->button, event->time);
    }
}
void dhms_time(unsigned int tis, char *d, char *h, char *m, char *s)         // converts seconds to days/hours/minutes/seconds
 {
    unsigned int tmp;
    
    *s = tis % 60;
    tmp = tis - *s;
    tmp = tmp / 60;
    *m = tmp % 60;
    tmp = tmp - *m;
    tmp = tmp / 60;
    *h = tmp % 24;
    tmp = tmp - *h;
    *d = tmp / 24;
}

unsigned int get_tracknumber(char *relpath)
{
  char fn_len, len;
  unsigned int pot, trackn;

  fn_len = strlen(relpath);
  pot = trackn = len = 0;

  relpath = relpath+fn_len-4;
  while ((fn_len != len) && (*relpath != '/'))
  {
    if ((*relpath >= 48) && (*relpath <= 57))
    {
      trackn = trackn + (*relpath-48) * pow(10, pot);
      pot = pot + 1;
    }
    relpath = relpath -1;
    len = len +1;
  }
  return trackn;
}

void delete_db_item()
{
  GtkCTree *tree = (GtkCTree *) ctree;
  additional_info *info;
  int seconds=0, songs=0;
  char d, h, m, s;

  if (!selected_tree_item) return;

  if ((GtkCTreeNode *) selected_tree_item == root) return; 
  else if ((info = gtk_ctree_node_get_row_data(GTK_CTREE(ctree), GTK_CTREE_NODE(selected_tree_item)))) {
    delete_song(tree, selected_tree_item, &seconds, TRUE); 
    songs = 1;
  }
  else if(GTK_CTREE_ROW( GTK_CTREE_ROW(selected_tree_item)->children )->children) { 
    songs = delete_artist(tree, selected_tree_item, &seconds);
  }
  else { 
    songs = delete_album(tree, selected_tree_item, &seconds);
  }
  selected_tree_item = NULL;
  
  dhms_time(seconds, &d, &h, &m, &s);
  printf("deleted %d songs (%02dd%02dh%02dm%02ds)\n", songs, d, h, m, s);
}

void delete_song(GtkCTree *tree, GtkCTreeNode *node, int *seconds, char free_ainfo)
{
  GtkCTreeNode *album, *artist;
  additional_info *ainfo;

  ainfo = gtk_ctree_node_get_row_data(GTK_CTREE(tree), node);
  album = GTK_CTREE_ROW(node)->parent;
  artist = GTK_CTREE_ROW(album)->parent;
  
  gtk_ctree_remove_node(GTK_CTREE(tree), node);
  if (seconds) *seconds = *seconds + ainfo->s_length;

  if (free_ainfo) {
  	// DON'T FREE THE ALBUM ! (part of node-text, for soundtracks only)
    free(ainfo->year);
    free(ainfo->comment);
    free(ainfo->filename);
    free(ainfo);
  }
  if (!GTK_CTREE_ROW(album)->children)
  {
    gtk_ctree_remove_node(GTK_CTREE(tree), album);
    if (!GTK_CTREE_ROW(artist)->children)
      gtk_ctree_remove_node(GTK_CTREE(tree), artist);
  }
}

int delete_album(GtkCTree *tree, GtkCTreeNode *node, int *seconds)
{
  GtkCTreeNode *song, *nsong;
  int songs=0;	// count deleted items

  song = GTK_CTREE_ROW(node)->children;
  if (song)
  {
    do {
      nsong = GTK_CTREE_ROW(song)->sibling;
      delete_song(tree, song, seconds, TRUE);
      songs++;
      song = nsong;
    } while (song);
  }
  return songs;
}

int delete_artist(GtkCTree *tree, GtkCTreeNode *node, int *seconds)
{
  GtkCTreeNode *album, *nalbum;
  int songs=0;

  album = GTK_CTREE_ROW(node)->children;
  if (album)
  {
    do {
      nalbum = GTK_CTREE_ROW(album)->sibling;
      songs += delete_album(tree, album, seconds);
      album = nalbum;
    } while (album);
  }
  return songs;
}

GtkCTreeNode *add_song_to_tree(GtkCTree *tree, GtkCTreeNode *rnode, char *artist, char *album, char *title, additional_info *ainfo)
{
  GtkCTreeNode *c_artist, *c_album, *c_song;
  char *text[2];

  text[1] = NULL;
  
  c_artist = my_find_tree_node(rnode, artist);
  if (!c_artist) {
     text[0] = artist;
     c_artist = gtk_ctree_insert_node(GTK_CTREE(tree), rnode, NULL, text, 5,
                       GTK_PIXMAP(artisticon)->pixmap, NULL,
                       GTK_PIXMAP(artisticon)->pixmap, NULL,
                       FALSE, FALSE);
  }

  c_album = my_find_tree_node(c_artist, album);
  if (!c_album)
  {
    text[0] = album;
    c_album = gtk_ctree_insert_node(GTK_CTREE(tree), c_artist, NULL, text, 5,
                       GTK_PIXMAP(albumicon)->pixmap, NULL,
                       GTK_PIXMAP(albumicon)->pixmap, NULL,
                       FALSE, FALSE);
  }

  text[0] = title;
  c_song = gtk_ctree_insert_node(GTK_CTREE(tree), c_album, NULL, text, 5,
  		GTK_PIXMAP(songicon)->pixmap, NULL,
		GTK_PIXMAP(songicon)->pixmap, NULL,
		FALSE, FALSE);
  gtk_ctree_node_set_row_data(GTK_CTREE(tree), c_song, ainfo);

  return (c_song);
}

void add_medium_to_node(GtkCTree *tree, GtkCTreeNode *node, char medium)
{
  GtkCTreeNode *c_artist, *c_album, *c_song;
  char *album, *artist, *title;
  additional_info *ainfo;

  if (!node)
    return;

  c_artist = GTK_CTREE_ROW(root)->children;
  do {
    artist = GTK_CELL_TEXT(GTK_CTREE_ROW(c_artist)->row.cell[0])->text;
    c_album = GTK_CTREE_ROW(c_artist)->children;
    do {
      album = GTK_CELL_TEXT(GTK_CTREE_ROW(c_album)->row.cell[0])->text;
      c_song = GTK_CTREE_ROW(c_album)->children;
      do {
        title = GTK_CELL_TEXT(GTK_CTREE_ROW(c_song)->row.cell[0])->text;
        ainfo = gtk_ctree_node_get_row_data(GTK_CTREE(tree), c_song);
	if (ainfo->onmedia == medium)
  	  add_song_to_tree(tree, node,
		artist, album, title, ainfo);
//	printf("%s - %s\n", artist, title);
	c_song = GTK_CTREE_ROW(c_song)->sibling;
      } while (c_song);
      c_album = GTK_CTREE_ROW(c_album)->sibling;
    } while (c_album);
    c_artist = GTK_CTREE_ROW(c_artist)->sibling;
  } while (c_artist);
  gtk_ctree_sort_recursive(tree, av_root);
}

void remove_medium_from_node(GtkCTree *tree, GtkCTreeNode *node, char medium)
{
  GtkCTreeNode *c_artist, *c_album, *c_song;
  GtkCTreeNode *nartist, *nalbum, *nsong;
  additional_info *ainfo;

  if (!node) return;
  c_artist = GTK_CTREE_ROW(node)->children;
  do {
    nartist = GTK_CTREE_ROW(c_artist)->sibling;
    c_album = GTK_CTREE_ROW(c_artist)->children;
    do {
      nalbum = GTK_CTREE_ROW(c_album)->sibling;
      c_song = GTK_CTREE_ROW(c_album)->children;
      do {
        nsong = GTK_CTREE_ROW(c_song)->sibling;
        ainfo = gtk_ctree_node_get_row_data(GTK_CTREE(tree), c_song);
	if (ainfo->onmedia == medium) {
	  delete_song(tree, c_song, NULL, FALSE);
	}
	c_song = nsong;
      } while (c_song);
      c_album = nalbum;
    } while (c_album);
    c_artist = nartist;
  } while (c_artist);
}
